/*=========================================================================
Class Name	: Application
Authors		: Ewan Duncan, Andrew Hoque
Description	: Main class of the program.  Contains the top
level program logic after main.cpp.
========================================================================*/

/*-------------------------------------------------------------------------
INCLUDES
-------------------------------------------------------------------------*/
#include "application.h"

#include "start_state.h"

/*-------------------------------------------------------------------------
CONSTRUCTORS
-------------------------------------------------------------------------*/
Application::Application()
{
}

/*-------------------------------------------------------------------------
DESTRUCTORS
-------------------------------------------------------------------------*/
Application::~Application()
{
}

/*-------------------------------------------------------------------------
MEMBER FUNCTIONS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Name	: Init
Purpose	: Initialises the application window and sets the initial state of
		  the program.  
-------------------------------------------------------------------------*/
void Application::Init()
{
	// Create the render window_.
	window_ = new sf::RenderWindow;

	window_->create(sf::VideoMode(kWindowWidth, kWindowHeight), "space budgie brief prototype");

	State::SetRenderWindow(window_);

	// set initial state
	state_manager_.ChangeState(new StartState());
}

/*-------------------------------------------------------------------------
Name	: CleanUp
Purpose	: Deletes the window.  
-------------------------------------------------------------------------*/
void Application::CleanUp()
{
	delete window_;
	window_ = NULL;
}

/*-------------------------------------------------------------------------
Name	: Update
Purpose	: Runs the main loop of the program.  
-------------------------------------------------------------------------*/
void Application::Update()
{
	// Set the frame rate to be consistant.
	window_->setVerticalSyncEnabled(true);
	
	while (window_->isOpen())
    {
		// There is an event every frame.
		sf::Event event;
		while (window_->pollEvent(event))
		{
			// Check event type.
			if (event.type == sf::Event::Closed)
			{
				window_->close();
			}
		}

		// update the input manager
		Services::Instance().GetInputManager().Update(0);

		// call appropriate state update function.
		state_manager_.states_vector.back()->Update(&state_manager_);

		// render the game
		Render();	

		// If escape is pressed, close the window/program.
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::Escape))
		{
			// close the window.
			window_->close();
		}

		// store the state of the input manager
		Services::Instance().GetInputManager().SetPreviousState();

		// update the clock and display the framerate
		clock_.Update();
		DisplayFramerate();
	}

	// call clean up.
	CleanUp();
}

/*-------------------------------------------------------------------------
Name	: Render
Purpose	: Renders the current state of the program.  
-------------------------------------------------------------------------*/
void Application::Render()
{
	// Clear the last frame.
	window_->clear();
	
	// call appropriate state render function.
	state_manager_.states_vector.back()->Render();	

	// Display this frame.
    window_->display();
}

/*-------------------------------------------------------------------------
Name	: DisplayFramerate
Purpose	: Outputs the frame rate.  
-------------------------------------------------------------------------*/
void Application::DisplayFramerate()
{
	const float time = 1.f / Clock::get_frame_time();
	std::stringstream stream;
	stream << "Current fps: " << time << std::endl;
	window_->setTitle(stream.str());
}

/*-------------------------------------------------------------------------
END OF FILE
-------------------------------------------------------------------------*/
