/*========================================================================
Class Name	: Boulder
Authors		: Andrew Hoque, Ewan Duncan, Lewis Bayne
Description	: Class that represents a boulder that can be pushed by the
player.
=========================================================================*/

/*-------------------------------------------------------------------------
INCLUDES
-------------------------------------------------------------------------*/
#include "boulder.h"
#include "game_state.h"
#include "services.h"
#include <iostream>

/*-------------------------------------------------------------------------
CONSTRUCTORS
-------------------------------------------------------------------------*/
Boulder::Boulder(const float width, const float height)
{
	// initialise boulder properties
	collidable_ = true;
	setSize(sf::Vector2f(width, height));
	setOrigin(getSize() * 0.5f);
	type_ = kBoulder;
	colliding_with_player_ = false;
	layer_ = 1;
	drawable_ = true;
	setTexture(Services::Instance().GetTextureManager().GetTexture("push_boulder.png"));
}


/*-------------------------------------------------------------------------
MEMBER FUNCTIONS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Name	: Update
Purpose	: Moves the boulder when it collides with the player and prevents 
		  it from moving through other boulders or collidable map objects.
-------------------------------------------------------------------------*/
void Boulder::Update()
{
	// store the boulder's current position in case the boulder collides with something and needs to be moved back
	sf::Vector2f temp_position = getPosition();

	// check if the boulder is colliding with the player
	if (colliding_with_player_)
	{
		// get a copy of the player from the vector of objects
		GameObject* object = Services::Instance().GetObjectsInUse()->at(0);
		int index = 0;

		// make sure that the object is the player and not another game object
		while (object->get_type() != kPlayer)
		{
			index++;
			object = Services::Instance().GetObjectsInUse()->at(index);
		}

		// cast the object as a player
		Player* player = reinterpret_cast<Player*>(object);

		// used to store the speed the boulder moves at
		const float speed = 225.0f;

		// slow the player down whilst it's pushing a boulder
		player->set_speed(speed);

		//// move the boulder based on the player's position in relation to the boulder ////

		if (player->getPosition().x > getPosition().x + (getSize().x * 0.5f))
		{
			MoveDeltaX(-speed * Clock::get_frame_time());
		}
		else if (player->getPosition().x < getPosition().x - (getSize().x * 0.5f))
		{
			MoveDeltaX(speed * Clock::get_frame_time());
		}
		else if (player->getPosition().y > getPosition().y + (getSize().y * 0.5f))
		{
			MoveDeltaY(-speed * Clock::get_frame_time());
		}
		else if (player->getPosition().y < getPosition().y - (getSize().y * 0.5f))
		{
			MoveDeltaY(speed * Clock::get_frame_time());
		}

		// play boulder pushing sound effect
		Services::Instance().GetSoundPlayer().set_position("pushing_boulder", getPosition().x, getPosition().y, 1.0f);
		Services::Instance().GetSoundPlayer().Play("pushing_boulder");
	}

	// loop through all the game objects in use
	for (int index = 0; index < Services::Instance().GetObjectsInUse()->size() - 1; index++)
	{
		// store the current game object
		GameObject* object = Services::Instance().GetObjectsInUse()->at(index);

		// if the object is this boulder then ignore it
		if (object == this)
		{
			continue;
		}

		// check if the object is another boulder or an orb
		if (object->get_type() == kBoulder || kOrb)
		{
			// see if the boulders are colliding
			if (CollisionDetection(*object))
			{
				// move the boulder back so it's not moving over the other boulder
				setPosition(temp_position);
			}
		}
	}

	// check for collisions with floating and pit collisions
	PitAndFloatingCollisions(temp_position);

	// check for collisions with the map
	MapCollision(temp_position);

	// the boulder is no longer colliding with the player
	colliding_with_player_ = false;
}

/*-------------------------------------------------------------------------
Name	: PitAndFloatingCollisions
Purpose	: Checks for when the boulder collides with a pit or water to 
		  prevent it from moving through them.  
-------------------------------------------------------------------------*/
void Boulder::PitAndFloatingCollisions(sf::Vector2f temp_position)
{
	// shift position to top left
	sf::Vector2f shifted_position = getPosition() - (getSize() * 0.5f);

	// get the map layers
	std::vector<tmx::MapLayer>& layers = map_loader_->GetLayers();
	
	// loop through all the map layers
	for (auto& layer : layers)
	{
		// if the layer is an object group
		if (layer.type == tmx::ObjectGroup)
		{
			// loop through all the objects on that layer
			for (auto& obj : layer.objects)
			{
				// if the layer is pit collisions or water collisions layer
				if (layer.name == "pit_collisions" || layer.name == "water_collisions")
				{
					// loop through all the points
					for (int point_index = 0; point_index < getPointCount(); point_index++)
					{
						// check for collision between the boulder and the points using the shifted position
						if (obj.Contains(shifted_position + getPoint(point_index)))
						{
							// move the boulder back so it's not moving over the pit or water
							setPosition(temp_position);
						}
					}
				}
			}
		}
	}
}

/*-------------------------------------------------------------------------
END OF FILE
-------------------------------------------------------------------------*/
