/*========================================================================
Class Name	: FireBall
Authors		: Ewan Duncan
Description	: Class that represents a fire ball that can be thrown by the
player and is used to burn webs.
=========================================================================*/

/*-------------------------------------------------------------------------
INCLUDES
-------------------------------------------------------------------------*/
#include "fire_ball.h"

#include "animated_prop.h"
#include "web.h"

/*-------------------------------------------------------------------------
CONSTRUCTORS
-------------------------------------------------------------------------*/
FireBall::FireBall(float x_position, float y_position, Direction direction)
{
	// initialise fire ball properties
	MoveTo(x_position, y_position);
	type_ = kFire;
	direction_ = direction;
	drawable_ = true;
	animation_finished_ = false;
	CreateLight(50.f, 10.f, Color3f(0.8f, 0.5f, 0.2f));
	TurnLightOn();

	// set up the texture and animation
	AnimatedPropInfo prop_info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("fire");
	end_uv_.x = prop_info.start_uv.x + ((prop_info.number_of_frames - 1)*  prop_info.dimensions.x);
	end_uv_.y = prop_info.start_uv.y;
	animation_map_["fly_forwards"] = new Animation(prop_info.start_uv, end_uv_, prop_info.dimensions, prop_info.animation_speed);
	setTexture(prop_info.texture);
	setSize(prop_info.dimensions);
	setOrigin(getSize() * 0.5f);
	setTextureRect(sf::IntRect(prop_info.start_uv.x, prop_info.start_uv.y, getSize().x, getSize().y));
}

/*-------------------------------------------------------------------------
MEMBER FUNCTIONS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Name	: Update
Purpose	: Moves the fire ball across the screen, animates it and checks for 
		  collisions with webs.  
-------------------------------------------------------------------------*/
void FireBall::Update()
{
	// make the light move with the fire ball
	light_->SetCenter(Vec2f(getPosition().x, kWindowHeight - getPosition().y));

	// check for when the animation has finished
	if (animation_finished_ == true)
	{
		// deactivate the fire ball
		active_ = false;
	}

	// change the uv coordinates to animate the fireball
	const sf::Vector2f new_uv = animation_map_["fly_forwards"]->animate();
	setTextureRect(sf::IntRect(new_uv.x, new_uv.y, getSize().x, getSize().y));

	// used to store the speed the fire ball moves at
	const float fire_speed = 300.f;

	///// move the fire ball based on the direction it was fired in ////

	switch (direction_)
	{
		case north:
			MoveDeltaY(-fire_speed * Clock::get_frame_time());
			break;
		case south:
			MoveDeltaY(fire_speed * Clock::get_frame_time());
			break;
		case west:
			MoveDeltaX(-fire_speed * Clock::get_frame_time());
			break;
		case east:
			MoveDeltaX(fire_speed * Clock::get_frame_time());
			break;
		case north_west:
			MoveDeltaX(-fire_speed * Clock::get_frame_time());
			MoveDeltaY(-fire_speed * Clock::get_frame_time());
			break;
		case north_east:
			MoveDeltaX(fire_speed * Clock::get_frame_time());
			MoveDeltaY(-fire_speed * Clock::get_frame_time());
			break;
		case south_west:
			MoveDeltaX(-fire_speed * Clock::get_frame_time());
			MoveDeltaY(fire_speed * Clock::get_frame_time());
			break;
		case south_east:
			MoveDeltaX(fire_speed * Clock::get_frame_time());
			MoveDeltaY(fire_speed * Clock::get_frame_time());
			break;
	}

	// loop through all the game objects in use
	for (int current_object = 0; current_object < Services::Instance().GetObjectsInUse()->size(); current_object++)
	{
		// see if the current object is a web
		if (Services::Instance().GetObjectsInUse()->at(current_object)->get_type() == kWeb)
		{
			// check if the fire ball is colliding with the web
			if (CollisionDetection(*Services::Instance().GetObjectsInUse()->at(current_object)) == true)
			{
				// set the web on fire
				reinterpret_cast<Web*>(Services::Instance().GetObjectsInUse()->at(current_object))->set_on_fire(true);

				// deactivate the fire ball
				active_ = false;
			}
		}
	}

	// use the UV coordinates to check when the animation is finished
	if (new_uv.x == end_uv_.x)
	{
		animation_finished_ = true;
	}
}

/*-------------------------------------------------------------------------
END OF FILE
-------------------------------------------------------------------------*/
