/*========================================================================
Class Name	: GameObject
Authors		: Ewan Duncan, Andrew Hoque
Description	: Class that represents an object in the game that gets updated
and rendered.
=========================================================================*/

/*-------------------------------------------------------------------------
INCLUDES
-------------------------------------------------------------------------*/
#include "game_object.h"

sf::Sprite* GameObject::world_sprite_;
sf::RenderTexture* GameObject::render_texture_;
tmx::MapLoader* GameObject::map_loader_;

/*-------------------------------------------------------------------------
CONSTRUCTORS
-------------------------------------------------------------------------*/
GameObject::GameObject() :
dead_only_(true),
light_(NULL)
{
	has_light_ = false;
	light_on_ = false;
	active_ = true;
}

/*-------------------------------------------------------------------------
DESTRUCTORS
-------------------------------------------------------------------------*/
GameObject::~GameObject()
{
	TurnLightOff();
}

/*-------------------------------------------------------------------------
MEMBER FUNCTIONS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Name	: Init
Purpose	: Initialises shared game object resources.
-------------------------------------------------------------------------*/
void GameObject::Init(sf::RenderTexture* render_texture, sf::Sprite* world_sprite, tmx::MapLoader* map_loader)
{
	render_texture_ = render_texture;
	world_sprite_ = world_sprite;
	map_loader_ = map_loader;
}

/*-------------------------------------------------------------------------
Name	: MoveTo
Purpose	: Moves the game object to a set specified position.
-------------------------------------------------------------------------*/
void GameObject::MoveTo(float xposition, float yposition)
{
	// move the game object to a specified position
	setPosition(sf::Vector2f(xposition, yposition));
}

/*-------------------------------------------------------------------------
Name	: SetX
Purpose	: Change the game object's x coordinate.
-------------------------------------------------------------------------*/
void GameObject::SetX(float xposition)
{
	// change the game object's x coordinate
	setPosition(sf::Vector2f(xposition, getPosition().y));
}

/*-------------------------------------------------------------------------
Name	: SetY
Purpose	: Change the game object's y coordinate.
-------------------------------------------------------------------------*/
void GameObject::SetY(float yposition)
{
	// change the game object's y coordinate
	setPosition(sf::Vector2f(getPosition().x, yposition));
}

/*-------------------------------------------------------------------------
Name	: MoveDeltaX
Purpose	: Move the game object in the x axis.
-------------------------------------------------------------------------*/
void GameObject::MoveDeltaX(float xdistance)
{
	// move the game object in the x axis
	setPosition(sf::Vector2f(getPosition().x + xdistance, getPosition().y));
}

/*-------------------------------------------------------------------------
Name	: MoveDeltaY
Purpose	: Move the game object in the y axis.
-------------------------------------------------------------------------*/
void GameObject::MoveDeltaY(float ydistance)
{
	// move the game object in the y axis
	setPosition(sf::Vector2f(getPosition().x, getPosition().y + ydistance));
}

/*-------------------------------------------------------------------------
Name	: CollisionDetection
Purpose	: Checks if the game object is colliding with another object.  
-------------------------------------------------------------------------*/
bool GameObject::CollisionDetection(GameObject& Object)
{
	// get the positions and half sizes of the two game objects
	float xposition1 = getPosition().x,
		  yposition1 = getPosition().y,
		  xposition2 = Object.getPosition().x,
		  yposition2 = Object.getPosition().y,
		  half_width1 = getSize().x * 0.5f,
		  half_height1 = getSize().y * 0.5f,
		  half_width2 = Object.getSize().x * 0.5f,
		  half_height2 = Object.getSize().y * 0.5f;

	// see if the first game object is to the left of the second game object
	if((xposition1 + half_width1) < (xposition2 - half_width2))
		return false;
	// see if the first game object is to the right of the second game object
	else if((xposition1 - half_width1) > (xposition2 + half_width2))
		return false;
	// see if the first game object is above the second game object
	else if((yposition1 + half_height1) < (yposition2 - half_height2))
		return false;
	// see if the first game object is below the second game object
	else if((yposition1 - half_height1) > (yposition2 + half_height2))
		return false;
	// if none of the previous conditions are true then the game objects must be colliding
	else
		return true;
}

/*-------------------------------------------------------------------------
Name	: Render
Purpose	: Renders the game object.
-------------------------------------------------------------------------*/
void GameObject::Render()
{
	if (drawable_ == true)
	{
		render_texture_->draw(*this);
	}	
}

/*-------------------------------------------------------------------------
Name	: CreateLight
Purpose	: Creates and initialises a light.  
-------------------------------------------------------------------------*/
void GameObject::CreateLight()
{
	light_ = new ltbl::Light_Point();

	light_->m_intensity = 2.0f;
	light_->m_center = Vec2f(getPosition().x, 720.0f - getPosition().y);
	light_->m_radius = 200.0f;
	light_->m_size = 1000.0f;
	light_->m_spreadAngle = ltbl::pifTimes2;
	light_->m_softSpreadAngle = 0.0f;
	light_->CalculateAABB();
	light_->m_bleed = 30.0f;
	light_->m_color = Color3f(0.0, 0.5, 1.0);
	light_->m_linearizeFactor = 0.2f;

	has_light_ = true;

	light_->SetAlwaysUpdate(true);
}

/*-------------------------------------------------------------------------
Name	: CreateLight
Purpose	: Creates and initialises a light using passed in parameters.
-------------------------------------------------------------------------*/
void GameObject::CreateLight(float radius, float bleed, Color3f colour)
{
	light_ = new ltbl::Light_Point();

	light_->m_intensity = 2.0f;
	light_->m_center = Vec2f(getPosition().x, 720.0f - getPosition().y);
	light_->m_radius = radius;
	light_->m_size = 1000.0f;
	light_->m_spreadAngle = ltbl::pifTimes2;
	light_->m_softSpreadAngle = 0.0f;
	light_->CalculateAABB();
	light_->m_bleed = bleed;
	light_->m_color = colour;
	light_->m_linearizeFactor = 0.2f;

	has_light_ = true;

	light_->SetAlwaysUpdate(true);
}

/*-------------------------------------------------------------------------
Name	: MapCollision
Purpose	: Checks for collisions with collidable objects on the map.  
-------------------------------------------------------------------------*/
void GameObject::MapCollision(sf::Vector2f temp_position)
{
	// shift position to top left
	sf::Vector2f shifted_position = getPosition() - (getSize() * 0.5f);

	// get the map layers
	std::vector<tmx::MapLayer>& layers = map_loader_->GetLayers();
	
	// loop through all the map layers
	for (auto& layer : layers)
	{
		// if the layer is an object group
		if (layer.type == tmx::ObjectGroup)
		{
			// loop through all the objects on that layer
			for (auto& obj : layer.objects)
			{
				// if the layer name is collisions
				if (layer.name == "collisions")
				{
					// loop through all the points
					for (int point_index = 0; point_index < getPointCount(); point_index++)
					{
						// check for collision between the game object and the points using the shifted position
						if (obj.Contains(shifted_position + getPoint(point_index)))
						{
							// move the game object back so it's not moving through the collidable object
							setPosition(temp_position);
						}
					}
				}
			}
		}
	}
}

/*-------------------------------------------------------------------------
Name	: TurnLightOn
Purpose	: Adds the game object's light to the light system
-------------------------------------------------------------------------*/
void GameObject::TurnLightOn()
{
	Services::Instance().GetLightSystem()->AddLight(light_);
}

/*-------------------------------------------------------------------------
Name	: TurnLightOff
Purpose	: Removes the game object's light from the light system
-------------------------------------------------------------------------*/
void GameObject::TurnLightOff()
{
	if (light_ != NULL)
	{
		Services::Instance().GetLightSystem()->RemoveLight(light_);
	}	
}

/*-------------------------------------------------------------------------
END OF FILE
-------------------------------------------------------------------------*/
