/*=========================================================================
Class Name	: GameState
Authors		: Ewan Duncan, Andrew Hoque
Description	: Contains in game logic.  Sets up tools used in
			  the game.  Manages the different sections(areas)
			  of the game world.
========================================================================*/

/*-------------------------------------------------------------------------
INCLUDES
-------------------------------------------------------------------------*/
#include "game_state.h"

#include <iostream>
using std::cout;
using std::endl;

#include <fstream>
#include <sstream>
using std::ifstream;

#include "services.h"
#include "clock.h"

/*-------------------------------------------------------------------------
MEMBER FUNCTIONS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Name	: Init
Purpose	: Initialises services managers, the light system, the render 
		  texture, the maploader and other properties.  
-------------------------------------------------------------------------*/
void GameState::Init()
{
	Services::Instance().GetTextureManager().LoadTextures();
	Services::Instance().GetAnimatedPropManager().LoadInfo();
	Services::Instance().GetMovingPropManager().LoadInfo();

	render_texture_.create(kWindowWidth, kWindowHeight);

	world_sprite_.setTexture(render_texture_.getTexture());

	light_system_ = new ltbl::LightSystem(AABB(Vec2f(0.0f, 0.0f), Vec2f(static_cast<float>(3840), static_cast<float>(3840))), window_, "data/lightFin.png", "data/shaders/lightAttenuationShader.frag");
	
	light_system_->m_ambientColor = sf::Color(25, 25, 25, 0);
	light_system_->m_useBloom = false;

	Services::Instance().SetLightSystem(light_system_);

	map_loader_ = new tmx::MapLoader("maps/");

	player_ = new Player(0.0f, 0.0f);

	LoadSections();

	GameObject::Init(&render_texture_, &world_sprite_, map_loader_);
}

/*-------------------------------------------------------------------------
Name	: CleanUp
Purpose	: Deletes objects no longer in use.  
-------------------------------------------------------------------------*/
void GameState::CleanUp()
{
	delete map_loader_;

	delete test_section_;

	delete player_;
}

/*-------------------------------------------------------------------------
Name	: Pause
Purpose	: Used to stop anything that needs stopped when the state is 
		  paused.  
-------------------------------------------------------------------------*/
void GameState::Pause()
{

}

/*-------------------------------------------------------------------------
Name	: Resume
Purpose	: Used to resume anything that was stopped when the state was
		  paused.
-------------------------------------------------------------------------*/
void GameState::Resume()
{

}

/*-------------------------------------------------------------------------
Name	: Update
Purpose	: Updates the section currently being used and the view.  
-------------------------------------------------------------------------*/
void GameState::Update(StateManager *state_manager)
{
	// update the current section
	if (section_map_[current_section_]->Update(current_section_) == true)
	{
		// if the current section changed, initialise the new section
		section_map_[current_section_]->Init();
	}

	// update the view
	ViewUpdate();

	// make the light system and render texture use the view
	light_system_->SetView(view_);
	render_texture_.setView(view_);
}

/*-------------------------------------------------------------------------
Name	: Render
Purpose	: Renders the game world.  
-------------------------------------------------------------------------*/
void GameState::Render()
{	
	// clear the render texture
	render_texture_.clear();

	// render the current section
	section_map_[current_section_]->Render();

	// Calculate the lights
	light_system_->RenderLights();

	// draw the lights
	light_system_->RenderLightTexture();
}

/*-------------------------------------------------------------------------
Name	: ViewUpdate
Purpose	: Makes the view follow the player whilst staying in the bounds
		  of the map.  
-------------------------------------------------------------------------*/
void GameState::ViewUpdate()
{
	// set the view position to the position of the player
	sf::Vector2f view_position = player_->getPosition();

	// move the view position to the corner of the window
	view_position.x -= kWindowWidth * 0.5f;
	view_position.y -= kWindowHeight * 0.5f;

	//// keep the view within the bounds of the map ////

	if (view_position.x > (map_loader_->GetMapSize().x - kWindowWidth))
		view_position.x = map_loader_->GetMapSize().x - kWindowWidth;

	if (view_position.x < 0)
		view_position.x = 0;

	if (view_position.y >(map_loader_->GetMapSize().y - kWindowHeight))
		view_position.y = map_loader_->GetMapSize().y - kWindowHeight;

	if (view_position.y < 0)
		view_position.y = 0;
	
	// reset the view using the view position and the window dimensions
	view_.reset(sf::FloatRect(view_position.x, view_position.y, kWindowWidth, kWindowHeight));
}

/*-------------------------------------------------------------------------
Name	: LoadSections
Purpose	: Loads the section maps in from a text file.  
-------------------------------------------------------------------------*/
void GameState::LoadSections()
{
	// create a file-reading object
	ifstream file;
	file.open("maps/sections.txt"); // open a file

	if (!file.good())
	{
		cout << "sections.txt not found";
	}

	std::string line = "";

	file.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
	file.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
	file.ignore(std::numeric_limits<std::streamsize>::max(), '\n');

	std::getline(file, line);

	std::string section_name = "";
	std::string dead_map_name = "";
	std::string alive_map_name = "";

	std::stringstream iss(line);
	getline(iss, section_name, ' ');
	getline(iss, dead_map_name, ' ');
	getline(iss, alive_map_name, ' ');

	section_map_[section_name] = new Section(dead_map_name, alive_map_name, map_loader_, player_, &render_texture_, &world_sprite_, window_);

	section_map_[section_name]->Init();

	current_section_ = section_name;

	// read each line of the file
	while (std::getline(file, line))
	{
		std::string section_name = "";
		std::string dead_map_name = "";
		std::string alive_map_name = "";

		std::stringstream iss(line);
		getline(iss, section_name, ' ');
		getline(iss, dead_map_name, ' ');
		getline(iss, alive_map_name, ' ');

		section_map_[section_name] = new Section(dead_map_name, alive_map_name, map_loader_, player_, &render_texture_, &world_sprite_, window_);
	}
}

/*-------------------------------------------------------------------------
END OF FILE
-------------------------------------------------------------------------*/
