/*========================================================================
Class Name	: InputManager
Authors		: Ewan Duncan
Description	: Class that encapsulates SFML's keyboard and add further
functionality.  Also adds Xbox 360 controller support.
=========================================================================*/

/*-------------------------------------------------------------------------
INCLUDES
-------------------------------------------------------------------------*/
#include "input_manager.h"

#ifdef __MINGW32__
    #define __in
    #define __out
    #define __reserved
#endif

#define WIN32_LEAN_AND_MEAN

/*-------------------------------------------------------------------------
MEMBER FUNCTIONS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Name	: IsControllerConnected
Purpose	: Checks if a specific controller is connected.  
-------------------------------------------------------------------------*/
bool InputManager::IsControllerConnected(t_controller_number controller_number)
{
	XINPUT_STATE state;
	ZeroMemory (&state, sizeof (XINPUT_STATE));

	auto result = XInputGetState(controller_number, &state);
	return  (result == ERROR_SUCCESS);
}

/*-------------------------------------------------------------------------
Name	: IsAnyXBox360ControllerConnected
Purpose	: Checks if any controllers are connected.
-------------------------------------------------------------------------*/
bool InputManager::IsAnyXBox360ControllerConnected()
{
	return  (IsControllerConnected(0) || IsControllerConnected(1) || IsControllerConnected(2) || IsControllerConnected(3));
}

/*-------------------------------------------------------------------------
Name	: IsButtonPressed
Purpose	: Checks if a button on a controller is pressed.  
-------------------------------------------------------------------------*/
bool InputManager::IsButtonPressed(t_controller_number controller_number, t_button_number button_number)
{
	bool button_pressed = false;

	if ((current_controller_state_.Gamepad.wButtons & button_number) != 0 && (previous_controller_state_.Gamepad.wButtons & button_number) == 0)
	{
		button_pressed = true;
	}

	return button_pressed;
}

/*-------------------------------------------------------------------------
Name	: IsButtonDown
Purpose	: Checks if a button on a controller is down.
-------------------------------------------------------------------------*/
bool InputManager::IsButtonDown(t_controller_number controller_number, t_button_number button_number)
{
	bool button_down = false;

	if (current_controller_state_.Gamepad.wButtons & button_number != 0)
	{
		button_down = true;
	}

	return button_down;
}

/*-------------------------------------------------------------------------
Name	: IsButtonReleased
Purpose	: Checks if a button on a controller is released.
-------------------------------------------------------------------------*/
bool InputManager::IsButtonReleased(t_controller_number controller_number, t_button_number button_number)
{
	bool button_released = false;

	if ((current_controller_state_.Gamepad.wButtons & button_number) == 0 && (previous_controller_state_.Gamepad.wButtons & button_number) != 0)
	{
		button_released = true;
	}

	return button_released;
}

/*-------------------------------------------------------------------------
Name	: IsKeyPressed
Purpose	: Checks if a key on the keyboard is pressed.
-------------------------------------------------------------------------*/
bool InputManager::IsKeyPressed(sf::Keyboard::Key key)
{
	if (sf::Keyboard::isKeyPressed(key) == true && previous_key_states_[key] == false)
	{
		return true;
	}
	else
	{
		return false;
	}
}

/*-------------------------------------------------------------------------
Name	: IsKeyDown
Purpose	: Checks if a key on the keyboard is down.
-------------------------------------------------------------------------*/
bool InputManager::IsKeyDown(sf::Keyboard::Key key)
{
	if (sf::Keyboard::isKeyPressed(key) == true)
	{
		return true;
	}
	else
	{
		return false;
	}
}

/*-------------------------------------------------------------------------
Name	: IsKeyReleased
Purpose	: Checks if a key on the keyboard is released.
-------------------------------------------------------------------------*/
bool InputManager::IsKeyReleased(sf::Keyboard::Key key)
{
	if (sf::Keyboard::isKeyPressed(key) == false && previous_key_states_[key] == true)
	{
		return true;
	}
	else
	{
		return false;
	}
}

/*-------------------------------------------------------------------------
Name	: SetCurrentState
Purpose	: Stores the current state of all the keys and buttons.  
-------------------------------------------------------------------------*/
void InputManager::SetCurrentState()
{
	ZeroMemory (&current_controller_state_, sizeof (XINPUT_STATE));
	XInputGetState (0, &current_controller_state_);

	for (int key_index = 0; key_index < sf::Keyboard::KeyCount; key_index++)
	{
		if (sf::Keyboard::isKeyPressed((sf::Keyboard::Key)key_index))
		{
			current_key_states_[key_index] = true;
		}
		else
		{
			current_key_states_[key_index] = false;
		}
	}
}

/*-------------------------------------------------------------------------
Name	: SetPreviousState
Purpose	: Stores the previous state of all the keys and buttons.
-------------------------------------------------------------------------*/
void InputManager::SetPreviousState()
{
	previous_controller_state_ = current_controller_state_;
	for (int key_index = 0; key_index < sf::Keyboard::KeyCount; key_index++)
	{
		previous_key_states_[key_index] = current_key_states_[key_index];
	}
}

/*-------------------------------------------------------------------------
Name	: SetTriggers
Purpose	: Stores the current state of a controllers' triggers.
-------------------------------------------------------------------------*/
void InputManager::SetTriggers(t_controller_number controller_number)
{
	XINPUT_STATE state;
	ZeroMemory (&state, sizeof (XINPUT_STATE));
	XInputGetState(controller_number, &state);

	left_trigger_  = static_cast <float> (state.Gamepad.bLeftTrigger)  / 255;
	right_trigger_ = static_cast <float> (state.Gamepad.bRightTrigger) / 255;
}

/*-------------------------------------------------------------------------
Name	: SetSticksPosition
Purpose	: Stores the current state of a controllers' analogue sticks.
-------------------------------------------------------------------------*/
void InputManager::SetSticksPosition(t_controller_number controller_number)
{
	XINPUT_STATE state;
	ZeroMemory (&state, sizeof (XINPUT_STATE));

	XInputGetState(controller_number, &state);

	if ( (state.Gamepad.sThumbLX < XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE &&
		  state.Gamepad.sThumbLX > -XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE) &&
		 (state.Gamepad.sThumbLY < XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE &&
		  state.Gamepad.sThumbLY > -XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE) ) 
	{
		state.Gamepad.sThumbLX = 0;
		state.Gamepad.sThumbLY = 0;
	}

	if ( (state.Gamepad.sThumbRX < XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE &&
		  state.Gamepad.sThumbRX > -XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE) &&
		 (state.Gamepad.sThumbRY < XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE &&
		  state.Gamepad.sThumbRY > -XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE) )
	{
		state.Gamepad.sThumbRX = 0;
		state.Gamepad.sThumbRY = 0;

	}

	left_stick_.x  = static_cast <float> (state.Gamepad.sThumbLX / 327);
	left_stick_.y  = static_cast <float> (state.Gamepad.sThumbLY / 327);
	right_stick_.x = static_cast <float> (state.Gamepad.sThumbRX / 327);
	right_stick_.y = static_cast <float> (state.Gamepad.sThumbRY / 327);
}

/*-------------------------------------------------------------------------
Name	: Update
Purpose	: Stores the current state of all the inputs.  
-------------------------------------------------------------------------*/
void InputManager::Update(t_controller_number controller_number)
{
	SetCurrentState();
	SetTriggers(controller_number);
	SetSticksPosition(controller_number);
}

/*-------------------------------------------------------------------------
Name	: SetVibration
Purpose	: Controls how much a controller vibrates
-------------------------------------------------------------------------*/
void InputManager::SetVibration(t_controller_number controller_number, float left_motor, float right_motor)
{
	XINPUT_VIBRATION vib;
	ZeroMemory (&vib, sizeof (XINPUT_VIBRATION));

	vib.wLeftMotorSpeed  = static_cast <WORD> (left_motor  * 65535.0f);
	vib.wRightMotorSpeed = static_cast <WORD> (right_motor * 65535.0f);

	XInputSetState(controller_number, &vib);
}

/*-------------------------------------------------------------------------
END OF FILE
-------------------------------------------------------------------------*/
