/*========================================================================
Class Name	: Map
Authors		: Ewan Duncan, Andrew Hoque
Description	: Works with the maploader to load maps that are created in Tiled.
			  Uses information from each map to create game entities and set 
			  properties such as music and lighting.
=========================================================================*/

/*-------------------------------------------------------------------------
INCLUDES
-------------------------------------------------------------------------*/

#include "map.h"

#include <sstream>
#include <stdlib.h> 
#include <string>

#include "orb.h"
#include "boulder.h"
#include "crystal_light.h"
#include "web.h"
#include "web.h"
#include "crystal_light.h"
#include "services.h"
#include "animated_prop.h"
#include "animated_tree.h"
#include "sound_trigger.h"
#include "moving_prop.h"

/*-------------------------------------------------------------------------
STATICS
-------------------------------------------------------------------------*/

std::string Map::map_type_ = "";

/*-------------------------------------------------------------------------
CONSTRUCTORS
-------------------------------------------------------------------------*/

Map::Map() :
visited_(false)
{
}

/*-------------------------------------------------------------------------
MEMBER FUNCTIONS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Name	: Init
Purpose	: Loads map and deals with spawning appropriate entities based on
		  layer information on the map. Also sets lighting and music for map.
		  Also turns on the lights for the map.
-------------------------------------------------------------------------*/
void Map::Init(tmx::MapLoader* map_loader, std::string map_name, std::vector<GameObject*>* section_objects)
{
	// start the timer to check how long the map takes to load
	Clock::StartTimer();

	// load the new map
	map_loader->Load(map_name);

	// get the layers vector
	std::vector<tmx::MapLayer>& layers = map_loader->GetLayers();

	// loop through each layer
	for (auto& layer : layers)
	{
		// if it is an object layer
		if (layer.type == tmx::ObjectGroup)
		{
			// check for objects on these layers
			for (auto& obj : layer.objects)
			{
				// Music
				if (layer.name == "music")
				{
					// if the type of map has changed, i.e. forestdead, forestalive
					if (obj.GetType() != map_type_)
					{
						// pause the current track that is playing
						Services::Instance().GetMusicPlayer().Pause();

						// play the new track taking information from the map
						Services::Instance().GetMusicPlayer().Play(obj.GetName());
					}

					// set code map type to the Tiled maps type
					map_type_ = obj.GetType();
				}

				// Ambient Lighting
				if (layer.name == "ambient_lighting")
				{
					// get the name of the object
					std::string ambient_values = obj.GetName();

					// floats used for RGB values of lighting
					float r, g, b;

					// token pointer
					char* endPtr;

					// get the RGB values, splitting up the string and converting to floats
					r = std::strtof(&ambient_values[0], &endPtr);
					g = std::strtof(endPtr, &endPtr);
					b = std::strtof(endPtr, &endPtr);

					// set the ambient lighting
					Services::Instance().GetLightSystem()->m_ambientColor = sf::Color(r, g, b);
				}
			}
		}
	}

	// stop the timer for loading the map
	std::cout << std::endl << "Took " << Clock::StopTimer() << " seconds to load" << std::endl;

	// if the map has never been visited by the player before
	if (visited_ == false)
	{
		std::vector<tmx::MapLayer>& layers = map_loader->GetLayers();
		for (auto& layer : layers)
		{
			if (layer.type == tmx::ObjectGroup)
			{
				for (auto& obj : layer.objects)
				{
					if (layer.name == "player_start")
					{
						// spawn the player at the position of the spawn object on the tiled map
						section_objects->at(0)->setPosition(obj.GetCentre());
					}

					if (layer.name == "orb_spawn")
					{
						// variables for animation and colour
						AnimatedPropInfo info;
						Color3f colour;

						// if there is nothing in the name field for the orb spawn object
						if (obj.GetName().size() == 0)
						{
							// default to this animation
							info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("orb_blue");
						}
						else
						{
							// otherwise get the animation that the designer wants
							info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo(obj.GetName());
						}

						// if there is no colour custom property
						if (obj.GetPropertyString("colour").size() == 0)
						{
							// default to this colour for the orb light
							colour = Color3f(0.8f, 0.3f, 0.8f);
						}
						else
						{
							// otherwise read in the data in the colour custom property
							std::string colour_values = obj.GetPropertyString("colour");

							// floats for RGB values
							float r, g, b;

							// token pointer
							char* endPtr;

							// split the string up and convert the values to floats
							r = std::strtof(&colour_values[0], &endPtr);
							g = std::strtof(endPtr, &endPtr);
							b = std::strtof(endPtr, &endPtr);

							// create colour object from these values
							colour = Color3f(r , g , b);
						}

						// set the power on the orb
						std::string power_name = obj.GetPropertyString("power");

						// add this orb to the section objects
						section_objects->push_back(new Orb(info, power_name, colour));

						// set position and whether or not object will only appear on dead maps
						SetPositionAndDeadOnly(obj, section_objects->back());
					}

					if (layer.name == "sound_triggers")
					{
						// get the dimensions of the object
						float width = (obj.GetCentre().x - obj.GetPosition().x) * 2;
						float height = (obj.GetCentre().y - obj.GetPosition().y) * 2;

						// read in whether or not the trigger will only ever trigger once
						std::string play_once_string = obj.GetPropertyString("play_once");

						// and if it will repeat when the player is inside the object
						std::string repeat_if_inside_string = obj.GetPropertyString("repeat_if_inside");

						// convert these to ints to be used as bools
						bool play_once = std::stoi(play_once_string);
						bool repeat_if_inside = std::stoi(repeat_if_inside_string);

						// read in the position that sound is to appear to be emitting from
						std::string position_string = obj.GetPropertyString("position");

						// check how long that is
						int string_size = position_string.length();

						// if something was there
						if (string_size != 0)
						{
							// token pointer
							char* endPtr;

							// read in and convert x and y position to floats
							float x = std::strtof(&position_string[0], &endPtr);
							float y = std::strtof(endPtr, NULL);

							// create a vector based on these values
							sf::Vector2f position = sf::Vector2f(x, y);

							// create a 3D sound triggers
							section_objects->push_back(new SoundTrigger(width, height, obj.GetName(), play_once, repeat_if_inside, position));
						}
						// if it is empty
						else if (string_size == 0)
						{
							// otherwise create a normal sound trigger
							section_objects->push_back(new SoundTrigger(width, height, obj.GetName(), play_once, repeat_if_inside));
						}

						// set position and whether or not object will only appear on dead maps
						SetPositionAndDeadOnly(obj, section_objects->back());
					}

					// legacy flower spawn
					if (layer.name == "flowers_spawn")
					{
						// hard coded legacy, delete this when designer fixes maps
						AnimatedPropInfo prop_info;

						prop_info.texture = Services::Instance().GetTextureManager().GetTexture("test_flower_animation_sprite.png");
						prop_info.start_uv = sf::Vector2f(0, 0);
				
						prop_info.number_of_frames = 11;
						prop_info.dimensions = sf::Vector2f(32, 64);
						prop_info.animation_speed = 35;
						prop_info.random_start = true;

						section_objects->push_back(new AnimatedProp(prop_info));
						SetPositionAndDeadOnly(obj, section_objects->back());
					}

					// legacy grass spawn
					if (layer.name == "grass_spawn")
					{
						// hard coded legacy, delete this when designer fixes maps
						AnimatedPropInfo prop_info;

						prop_info.texture = Services::Instance().GetTextureManager().GetTexture("LiveSprites.png");
						prop_info.start_uv = sf::Vector2f(0, 0);
						prop_info.number_of_frames = 32;
						prop_info.dimensions = sf::Vector2f(128, 128);
						prop_info.animation_speed = 70;
						prop_info.random_start = true;

						section_objects->push_back(new AnimatedProp(prop_info));
						SetPositionAndDeadOnly(obj, section_objects->back());
					}

					if (layer.name == "animations")
					{
						// check the name of the object and get animation data based on this
						AnimatedPropInfo info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo(obj.GetName());

						// read in custom property layer
						std::string layer_number = obj.GetPropertyString("layer");

						// if there was something there
						if (layer_number.size() != 0 && layer_number != " ")
						{
							// override the default layer in the animation with this
							info.layer_number = std::stoi(layer_number);
						}

						section_objects->push_back(new AnimatedProp(info));

						// set position and whether or not object will only appear on dead maps
						SetPositionAndDeadOnly(obj, section_objects->back());
					}

					if (layer.name == "tree_spawn")
					{
						// get animation data for tree2
						AnimatedPropInfo info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("tree2");

						// get custom property for distance
						std::string trigger_distance = obj.GetPropertyString("distance");

						// if nothing was there
						if (trigger_distance.size() == 0)
						{
							// default to 500 units
							section_objects->push_back(new AnimatedTree(info, 500));
						}
						else
						{
							// otherwise change the trigger distance to the one that is on the object
							section_objects->push_back(new AnimatedTree(info, std::stoi(trigger_distance)));
						}

						// set position and whether or not object will only appear on dead maps
						SetPositionAndDeadOnly(obj, section_objects->back());
					}

					if (layer.name == "moving_props")
					{
						// get data for where the prop is moving to
						std::string destination_string = obj.GetPropertyString("destination");

						// get data for how fast the prop should move toward this destination
						std::string speed_string = obj.GetPropertyString("speed");

						// whether or not the prop will begin its idle or dissapear when it reaches destination
						std::string stay_string = obj.GetPropertyString("stay");

						// the name of the prop to be used
						std::string prop_name = obj.GetName();

						// token pointer
						char* endPtr;

						// split the destination string and convert to floats
						float x = std::strtof(&destination_string[0], &endPtr);
						float y = std::strtof(endPtr, NULL);

						// create destination vector based on this
						sf::Vector2f destination = sf::Vector2f(x, y);

						// get the radius of the circle used to create the object, will be used as trigger distance
						float radius = obj.GetCentre().x - obj.GetPosition().x;

						// convert speed string to float
						float speed = std::strtof(&speed_string[0], NULL);

						// convert stay string to int, for use as bool
						bool stay = std::stoi(stay_string);

						// get the prop data based on the prop name
						MovingPropInfo info = *Services::Instance().GetMovingPropManager().GetMovingPropInfo(prop_name);

						// add to vector for this section
						section_objects->push_back(new MovingProp(radius , speed , stay , destination,info));

						// set position and whether or not object will only appear on dead maps
						SetPositionAndDeadOnly(obj, section_objects->back());
					}

					if (layer.name == "lights")
					{
						// get the colour custom property
						std::string colour_values = obj.GetPropertyString("colour");

						// floats for RGB values of the colour
						float r, g, b;

						// token pointer
						char* endPtr;

						// split the colour values string up and comvert to floats
						r = std::strtof(&colour_values[0], &endPtr);
						g = std::strtof(endPtr, &endPtr);
						b = std::strtof(endPtr, &endPtr);

						// create colour vector based on these
						Color3f colour = Color3f(r, g, b);

						// get the bleed property and convert to float
						std::string bleed_value = obj.GetPropertyString("bleed");

						float bleed = std::strtof(&bleed_value[0], NULL);

						// get the radius of the light and convert to float
						std::string radius_value = obj.GetPropertyString("radius");

						float radius = std::strtof(&radius_value[0], NULL);

						// add the light to section objects
						section_objects->push_back(new GameObject);

						// set type
						section_objects->back()->set_type(kLight);

						// set position and whether or not object will only appear on dead maps
						SetPositionAndDeadOnly(obj, section_objects->back());

						// actually set the light objects light pointer with the LTBL system
						section_objects->back()->CreateLight(radius, bleed, colour);
					}

					if (layer.name == "crystal_light")
					{
						// get the colour property for the light
						std::string colour_values = obj.GetPropertyString("colour");

						// RGB value floats
						float r, g, b;

						// token pointer
						char* endPtr;

						// split the colour string up and convert to floats
						r = std::strtof(&colour_values[0], &endPtr);
						g = std::strtof(endPtr, &endPtr);
						b = std::strtof(endPtr, &endPtr);

						// create colour objetc based on these
						Color3f colour = Color3f(r, g, b);

						// get the bleed property and convert to float
						std::string bleed_value = obj.GetPropertyString("bleed");

						float bleed = std::strtof(&bleed_value[0], NULL);

						// get the radius of the light and convert to float
						std::string radius_value = obj.GetPropertyString("radius");

						float radius = std::strtof(&radius_value[0], NULL);

						// get the distance that the light will trigger at 
						float distance = obj.GetCentre().x - obj.GetPosition().x;
						
						section_objects->push_back(new Crystal_Light(distance, radius));
		
						// set position and whether or not object will only appear on dead maps
						SetPositionAndDeadOnly(obj, section_objects->back());

						// actually set the light objects light pointer with the LTBL system
						section_objects->back()->CreateLight(radius, bleed, colour);					
					}

					if (layer.name == "boulder")
					{
						// get the dimensions of the objects
						float width = (obj.GetCentre().x - obj.GetPosition().x) * 2;
						float height = (obj.GetCentre().y - obj.GetPosition().y) * 2;

						section_objects->push_back(new Boulder(width, height));

						// set position and whether or not object will only appear on dead maps
						SetPositionAndDeadOnly(obj, section_objects->back());
					}

					if (layer.name == "webs")
					{
						// get the dimensions of the objects
						float width = (obj.GetCentre().x - obj.GetPosition().x) * 2;
						float height = (obj.GetCentre().y - obj.GetPosition().y) * 2;

						section_objects->push_back(new Web(width, height));

						// set position and whether or not object will only appear on dead maps
						SetPositionAndDeadOnly(obj, section_objects->back());
					}
				}
			}
		}

		// player has now visited this map, so this section of code wont be called again for this map
		visited_ = true;
	}

	// turn the lights on in the section
	TurnOnLights(section_objects);
}

/*-------------------------------------------------------------------------
Name	: SetPositionAndDeadOnly
Purpose	: Sets the position of a game object based on its position on a Tiled
		  map. Also sets whether or not it should appear only on dead maps
		  based on a property on the object in Tiled.
-------------------------------------------------------------------------*/
void Map::SetPositionAndDeadOnly(tmx::MapObject map_object, GameObject* section_object)
{
	// set the position of the object based on the centre of the object created on the Tiled map
	section_object->setPosition(map_object.GetCentre());

	// get the dead_only property of the object
	std::string dead_only = map_object.GetPropertyString("dead_only");

	// if something is actually there
	if (dead_only.size() != 0)
	{
		// set whether or not the object only appears in dead maps, converting to int to be used as a bool
		section_object->set_dead_only(std::stoi(dead_only));
	}
}

/*-------------------------------------------------------------------------
Name	: TurnOnLights
Purpose	: Checks if objects on this map have lights. If they do, it turns
		  them on.
-------------------------------------------------------------------------*/
void Map::TurnOnLights(std::vector<GameObject*>* section_objects)
{
	// loop through all objects in the section
	for (int current_object = 0; current_object < section_objects->size(); current_object++)
	{
		// if the object has a light
		if ((*section_objects)[current_object]->get_has_light() == true)
		{
			// turn it on
			(*section_objects)[current_object]->TurnLightOn();
		}
	}
}

/*-------------------------------------------------------------------------
END OF FILE
-------------------------------------------------------------------------*/
