/*============================================================
Class Name	: Player
Authors		: Ewan Duncan, Lewis Bayne, Andrew Hoque
Description	: Object that represents the player in the game.
Takes input from user and acts on it.
============================================================*/

/*------------------------------------------------------------
INCLUDES
------------------------------------------------------------*/
#include "player.h"

#include <cmath>
#include <iostream>

#include "services.h"
#include "clock.h"
#include "boulder.h"
#include "fire_ball.h"
#include "animated_prop.h"

/*-------------------------------------------------------------------------
CONSTRUCTORS
-------------------------------------------------------------------------*/
Player::Player(float x, float y) :
push_boulder_power_(false),
float_power_(false),
pit_float_power_(false),
fire_power_(false)
{
	// initialise player properties
	MoveTo(x, y);
	setSize(sf::Vector2f(96, 128));
	setOrigin(getSize() * 0.5f);
	type_ = kPlayer;
	direction_ = south;
	drawable_ = true;
	moving_ = false;
	controllable_ = true;
	floating_ = false;
	dead_only_ = false;
	fire_ball_limiter_ = 0;
	just_obtained_fire_ = false;

	// set up the texture and animations
	SetUVCoordinates(0, 0);
	
	setTexture(Services::Instance().GetTextureManager().GetTexture("player_walk"));

	AnimatedPropInfo info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("player_walk_up");

	animation_map_["walk_up"] = new Animation(info.start_uv, info.number_of_frames, info.dimensions, info.animation_speed);

	info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("player_walk_down");
	animation_map_["walk_down"] = new Animation(info.start_uv, info.number_of_frames, info.dimensions, info.animation_speed);

	info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("player_walk_left");
	animation_map_["walk_left"] = new Animation(info.start_uv, info.number_of_frames, info.dimensions, info.animation_speed);

	info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("player_walk_right");
	animation_map_["walk_right"] = new Animation(info.start_uv, info.number_of_frames, info.dimensions, info.animation_speed);

	info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("player_float_up");
	animation_map_["float_up"] = new Animation(info.start_uv, info.number_of_frames, info.dimensions, info.animation_speed);

	info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("player_float_down");
	animation_map_["float_down"] = new Animation(info.start_uv, info.number_of_frames, info.dimensions, info.animation_speed);

	info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("player_float_left");
	animation_map_["float_left"] = new Animation(info.start_uv, info.number_of_frames, info.dimensions, info.animation_speed);

	info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("player_float_right");
	animation_map_["float_right"] = new Animation(info.start_uv, info.number_of_frames, info.dimensions, info.animation_speed);

	const sf::Vector2f new_uv = animation_map_["walk_up"]->animate();

	setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 175, 265));

	LoadProperties();
}

/*-------------------------------------------------------------------------
DESTRUCTORS
-------------------------------------------------------------------------*/
Player::~Player()
{
	
}

/*-------------------------------------------------------------------------
MEMBER FUNCTIONS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Name	: LoadProperties
Purpose	: Loads player properties from a text file.  
-------------------------------------------------------------------------*/
void Player::LoadProperties()
{
	const std::string file_path = "media/";

	// create a file-reading object
	ifstream file;
	file.open(file_path + "player.txt"); // open a file

	if (!file.good())
	{
		cout << "player.txt not found";
	}

	std::string line = "";

	file.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
	file.ignore(std::numeric_limits<std::streamsize>::max(), '\n');

	std::getline(file, line);

	std::string layer_number = "";
	std::string default_speed = "";

	std::stringstream iss(line);
	getline(iss, layer_number, ' ');
	getline(iss, default_speed, ' ');

	layer_ = std::stoi(layer_number);
	default_speed_ = std::stoi(default_speed);

	std::cout << "player layer: " << layer_ << std::endl;
}

/*-------------------------------------------------------------------------
Name	: Input
Purpose	: Handles inputs from the user.  
-------------------------------------------------------------------------*/
void Player::Input()
{
	// get the input manager from services
	InputManager input_manager = Services::Instance().GetInputManager();

	const float rate_of_fire = 0.5f;

	// check if the player has the power to throw fire balls
	if (fire_power_ == true)
	{
		if (just_obtained_fire_ == true)
		{
			Services::Instance().GetObjectsInUse()->push_back(new FireBall(getPosition().x, getPosition().y, direction_));
			just_obtained_fire_ = false;
		}

		fire_ball_limiter_ += Clock::get_frame_time();

		// when the spacebar or the A button is pressed, create a new fire ball
		if (input_manager.IsKeyPressed(sf::Keyboard::Space) || input_manager.IsButtonPressed(0, InputManager::A))

		{
			
			if(fire_ball_limiter_ >= rate_of_fire)
			{
				Services::Instance().GetObjectsInUse()->push_back(new FireBall(getPosition().x, getPosition().y, direction_));

				fire_ball_limiter_ = 0;
			}
		}

	}

	// the player is moving if any of the arrow keys or WASD keys are held down or if the left stick is moved
	if (input_manager.IsKeyDown(sf::Keyboard::Left) || input_manager.IsKeyDown(sf::Keyboard::Right) ||
		input_manager.IsKeyDown(sf::Keyboard::Up) || input_manager.IsKeyDown(sf::Keyboard::Down) ||
		input_manager.IsKeyDown(sf::Keyboard::A) || input_manager.IsKeyDown(sf::Keyboard::D) ||
		input_manager.IsKeyDown(sf::Keyboard::W) || input_manager.IsKeyDown(sf::Keyboard::S) ||
		input_manager.get_left_stick_position().x != 0 || input_manager.get_left_stick_position().y != 0)
	{
		moving_ = true;

		if (!floating_)
		{
			Services::Instance().GetSoundPlayer().Play("player_footsteps");
		}	
	}

	//// move the player in the direction appropriate for the current input ////

	if ((input_manager.IsKeyDown(sf::Keyboard::Left) && input_manager.IsKeyDown(sf::Keyboard::Up)) ||
		(input_manager.IsKeyDown(sf::Keyboard::A) && input_manager.IsKeyDown(sf::Keyboard::W)) ||
		(input_manager.get_left_stick_position().x < -45 && input_manager.get_left_stick_position().y > 45))
	{
		direction_ = north_west;
		MoveNorthWest();
	}
	else if ((input_manager.IsKeyDown(sf::Keyboard::Right) && input_manager.IsKeyDown(sf::Keyboard::Up)) ||
			 (input_manager.IsKeyDown(sf::Keyboard::D) && input_manager.IsKeyDown(sf::Keyboard::W)) ||
			 (input_manager.get_left_stick_position().x > 45 && input_manager.get_left_stick_position().y > 45))
	{
		direction_ = north_east;
		MoveNorthEast();
	}
	else if ((input_manager.IsKeyDown(sf::Keyboard::Left) && input_manager.IsKeyDown(sf::Keyboard::Down)) ||
			 (input_manager.IsKeyDown(sf::Keyboard::A) && input_manager.IsKeyDown(sf::Keyboard::S)) ||
			 (input_manager.get_left_stick_position().x < -45 && input_manager.get_left_stick_position().y < -45))
	{
		direction_ = south_west;
		MoveSouthWest();
	}
	else if ((input_manager.IsKeyDown(sf::Keyboard::Right) && input_manager.IsKeyDown(sf::Keyboard::Down)) ||
			 (input_manager.IsKeyDown(sf::Keyboard::D) && input_manager.IsKeyDown(sf::Keyboard::S)) ||
			 (input_manager.get_left_stick_position().x > 45 && input_manager.get_left_stick_position().y < -45))
	{
		direction_ = south_east;
		MoveSouthEast();
	}
	else if (input_manager.IsKeyDown(sf::Keyboard::W) || input_manager.IsKeyDown(sf::Keyboard::Up) ||
			 input_manager.get_left_stick_position().y > 80)
	{
		direction_ = north;

		MoveUp();
	}
	else if (input_manager.IsKeyDown(sf::Keyboard::S) || input_manager.IsKeyDown(sf::Keyboard::Down) ||
			 input_manager.get_left_stick_position().y < -80)
	{
		direction_ = south;

		MoveDown();
	}
	else if (input_manager.IsKeyDown(sf::Keyboard::A) || input_manager.IsKeyDown(sf::Keyboard::Left) ||
			 input_manager.get_left_stick_position().x < -80)
	{
		direction_ = west;

		MoveLeft();
	}
	else if (input_manager.IsKeyDown(sf::Keyboard::D) || input_manager.IsKeyDown(sf::Keyboard::Right) ||
			 input_manager.get_left_stick_position().x > 80)
	{
		direction_ = east;

		MoveRight();
	}
}

/*-------------------------------------------------------------------------
Name	: Update
Purpose	: Checks for collisions with other objects and calls funtions for
		  handling inputs and animating the player.  
-------------------------------------------------------------------------*/
void Player::Update()
{
	// have the listener follow the player
	sf::Listener::setPosition(getPosition().x, getPosition().y, 1.0f);

	// store the player's current position in case the player collides with something and needs to be moved back
	sf::Vector2f temp_position = getPosition();

	// by default, the player doesn't move
	moving_ = false;

	// check that the player is currently controllable
	if (controllable_ == true)
	{
		// handle input for the player
		Input();
	}
	
	// when the player isn't moving, it is idle
	if (!moving_ && controllable_ == true)
	{
		//Idle();
	}

	if (!floating_)
	{
		setTexture(Services::Instance().GetTextureManager().GetTexture("player_walk"));
		setSize(sf::Vector2f(96, 128));
	}
	else
	{
		setTexture(Services::Instance().GetTextureManager().GetTexture("player_float"));
		setSize(sf::Vector2f(96, 140));
	}

	// loop through all the game objects in use
	for (int current_object = 0; current_object < Services::Instance().GetObjectsInUse()->size(); current_object++)
	{
		// see if the current object is a boulder
		if (Services::Instance().GetObjectsInUse()->at(current_object)->get_type() == kBoulder)
		{
			// check if the player is colliding with the boulder
			if (CollisionDetection(*Services::Instance().GetObjectsInUse()->at(current_object)) == true)
			{
				// move the player back so it's not moving over the boulder
				setPosition(temp_position);

				// if the player has the power to push boulders
				if (push_boulder_power_)
				{
					// set the boulder to be colliding with the player
					reinterpret_cast<Boulder*>(Services::Instance().GetObjectsInUse()->at(current_object))->set_colliding_with_player(true);
				}
			}
		}

		// see if the current object is a web
		if (Services::Instance().GetObjectsInUse()->at(current_object)->get_type() == kWeb)
		{
			// check if the player is colliding with the web
			if (CollisionDetection(*Services::Instance().GetObjectsInUse()->at(current_object)) == true)
			{
				// move the player back so it's not moving over the web
				setPosition(temp_position);
			}
		}
	}

	// check for collisions with the map
	MapCollision(temp_position);

	// check for collisions with objects that can be floated over
	FloatingCollisions(temp_position);

	// set the default player speed
	speed_ = default_speed_;
}

/*-------------------------------------------------------------------------
Name	: WalkThroughDoor
Purpose	: Makes the player slowly walk through a door into a new section.  
-------------------------------------------------------------------------*/
void Player::WalkThroughDoor(std::string direction)
{
	// slow the player down when walking through a door
	speed_ = 150.f;

	///// move the player based on the direction they are facing ////

	if (direction == "up")
	{
		MoveDown();
	}
	else if (direction == "down")
	{
		MoveUp();
	}
	else if (direction == "left")
	{
		MoveRight();
	}
	else if (direction == "right")
	{
		MoveLeft();
	}
}

/*-------------------------------------------------------------------------
Name	: distanceChange
Purpose	: Changes the colour of the player based on the distance to another
		  game object.  
-------------------------------------------------------------------------*/
void Player::distanceChange(GameObject& Object)
{
	// use pythagoras to get the distance between the player and the object in question
	float distance = sqrt((getPosition().x - Object.getPosition().x)*(getPosition().x - Object.getPosition().x) + (getPosition().y - Object.getPosition().y)*(getPosition().y - Object.getPosition().y));

	//// change the colour based on the distance ////

	if (distance <= 800)
	{
		setFillColor(sf::Color::Red);
	}
	if (distance <= 600)
	{
		setFillColor(sf::Color::Yellow);
	}
	if (distance <= 300)
	{
		setFillColor(sf::Color::Green);
	}
}

/*-------------------------------------------------------------------------
Name	: SetUVCoordinates
Purpose	: Changes the UV coordinates of the player.  
-------------------------------------------------------------------------*/
void Player::SetUVCoordinates(float x, float y)
{
	// change the texture rectangle based on the UV coordinates
	setTextureRect(sf::IntRect(x, y, 294.5, 438));
}

/*-------------------------------------------------------------------------
Name	: Idle
Purpose	: Changes between the different idle animations when the player is
		  stationary.  
-------------------------------------------------------------------------*/
void Player::Idle()
{
	//// change the idle animation based on the direction the player is facing and wether or not they are floating ////

	switch (direction_)
	{
		case north_west:
			if (!floating_)
			{
				//const sf::Vector2f new_uv = walk_down_->animate();
				SetUVCoordinates(0, 0);
			}
			else
			{
				//const sf::Vector2f new_uv = walk_down_->animate();
				SetUVCoordinates(0, 0);
			}
			break;
		case north_east:
			if (!floating_)
			{
				//const sf::Vector2f new_uv = walk_down_->animate();
				SetUVCoordinates(0, 0);
			}
			else
			{
				//const sf::Vector2f new_uv = walk_down_->animate();
				SetUVCoordinates(0, 0);
			}
			break;
		case south_west:
			if (!floating_)
			{
				//const sf::Vector2f new_uv = walk_up_->animate();
				SetUVCoordinates(0, 0);
			}
			else
			{
				//const sf::Vector2f new_uv = walk_up_->animate();
				SetUVCoordinates(0, 0);
			}

			break;
		case south_east:
			if (!floating_)
			{
				//const sf::Vector2f new_uv = walk_up_->animate();
				SetUVCoordinates(0, 0);
			}
			else
			{
				//const sf::Vector2f new_uv = walk_up_->animate();
				SetUVCoordinates(0, 0);
			}
			break;
		case north:
			if (!floating_)
			{
				//const sf::Vector2f new_uv = walk_down_->animate();
				SetUVCoordinates(0, 0);
			}
			else
			{
				//const sf::Vector2f new_uv = walk_down_->animate();
				SetUVCoordinates(0, 0);
			}
			break;
		case south:
			if (!floating_)
			{
				//const sf::Vector2f new_uv = walk_up_->animate();
				SetUVCoordinates(0, 0);
			}
			else
			{
				//const sf::Vector2f new_uv = walk_up_->animate();
				SetUVCoordinates(0, 0);
			}
			break;
		case west:
			if (!floating_)
			{
				//const sf::Vector2f new_uv = walk_right_->animate();
				SetUVCoordinates(0, 0);
			}
			else
			{
				//const sf::Vector2f new_uv = walk_right_->animate();
				SetUVCoordinates(0, 0);
			}
			break;
		case east:
			if (!floating_)
			{
				//const sf::Vector2f new_uv = walk_left_->animate();
				SetUVCoordinates(0, 0);
			}
			else
			{
				//const sf::Vector2f new_uv = walk_left_->animate();
				SetUVCoordinates(0, 0);
			}
			break;
	}
}

/*-------------------------------------------------------------------------
Name	: MoveNorthWest
Purpose	: Moves the player northwest and plays the appropriate animation.  
-------------------------------------------------------------------------*/
void Player::MoveNorthWest()
{
	// move the player northwest
	MoveDeltaY(-speed_ * Clock::get_frame_time());
	MoveDeltaX(-speed_ * Clock::get_frame_time());

	// change the animation based on wether or not the player is floating
	if (!floating_)
	{
		const sf::Vector2f new_uv = animation_map_["walk_up"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 175, 265));
	}
	else
	{
		const sf::Vector2f new_uv = animation_map_["float_up"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 175, 286));
	}
}

/*-------------------------------------------------------------------------
Name	: MoveNorthEast
Purpose	: Moves the player northeast and plays the appropriate animation.
-------------------------------------------------------------------------*/
void Player::MoveNorthEast()
{
	// move the player northeast
	MoveDeltaY(-speed_ * Clock::get_frame_time());
	MoveDeltaX(speed_ * Clock::get_frame_time());

	// change the animation based on wether or not the player is floating
	if (!floating_)
	{
		const sf::Vector2f new_uv = animation_map_["walk_up"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 175, 265));
	}
	else
	{
		const sf::Vector2f new_uv = animation_map_["float_up"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 175, 286));
	}
}

/*-------------------------------------------------------------------------
Name	: MoveSouthWest
Purpose	: Moves the player southwest and plays the appropriate animation.
-------------------------------------------------------------------------*/
void Player::MoveSouthWest()
{
	// move the player southwest
	MoveDeltaY(speed_ * Clock::get_frame_time());
	MoveDeltaX(-speed_ * Clock::get_frame_time());

	// change the animation based on wether or not the player is floating
	if (!floating_)
	{
		const sf::Vector2f new_uv = animation_map_["walk_down"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 189.275, 250));
	}
	else
	{
		const sf::Vector2f new_uv = animation_map_["float_down"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 190, 270));
	}
}

/*-------------------------------------------------------------------------
Name	: MoveSouthEast
Purpose	: Moves the player southeast and plays the appropriate animation.
-------------------------------------------------------------------------*/
void Player::MoveSouthEast()
{
	// move the player south east
	MoveDeltaY(speed_ * Clock::get_frame_time());
	MoveDeltaX(speed_ * Clock::get_frame_time());

	// change the animation based on wether or not the player is floating
	if (!floating_)
	{
		const sf::Vector2f new_uv = animation_map_["walk_down"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 189.275, 250));
	}
	else
	{
		const sf::Vector2f new_uv = animation_map_["float_down"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 190, 270));
	}
}

/*-------------------------------------------------------------------------
Name	: MoveUp
Purpose	: Moves the player up and plays the appropriate animation.
-------------------------------------------------------------------------*/
void Player::MoveUp()
{
	// move the player north
	MoveDeltaY(-speed_ * Clock::get_frame_time());

	// change the animation based on wether or not the player is floating
	if (!floating_)
	{
		const sf::Vector2f new_uv = animation_map_["walk_up"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 175, 265));
	}
	else
	{
		const sf::Vector2f new_uv = animation_map_["float_up"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 175, 286));
	}
}

/*-------------------------------------------------------------------------
Name	: MoveDown
Purpose	: Moves the player down and plays the appropriate animation.
-------------------------------------------------------------------------*/
void Player::MoveDown()
{
	// move the player south
	MoveDeltaY(speed_ * Clock::get_frame_time());

	// change the animation based on wether or not the player is floating
	if (!floating_)
	{
		const sf::Vector2f new_uv = animation_map_["walk_down"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 189.275, 250));
	}
	else
	{
		const sf::Vector2f new_uv = animation_map_["float_down"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 190, 270));
	}
}

/*-------------------------------------------------------------------------
Name	: MoveLeft
Purpose	: Moves the player left and plays the appropriate animation.
-------------------------------------------------------------------------*/
void Player::MoveLeft()
{
	// move the player west
	MoveDeltaX(-speed_ * Clock::get_frame_time());

	// change the animation based on wether or not the player is floating
	if (!floating_)
	{
		const sf::Vector2f new_uv = animation_map_["walk_left"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 160, 250));
	}
	else
	{
		const sf::Vector2f new_uv = animation_map_["float_left"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 160, 283));
	}
}

/*-------------------------------------------------------------------------
Name	: MoveRight
Purpose	: Moves the player right and plays the appropriate animation.
-------------------------------------------------------------------------*/
void Player::MoveRight()
{
	// move the player east
	MoveDeltaX(speed_ * Clock::get_frame_time());

	// change the animation based on wether or not the player is floating
	if (!floating_)
	{
		const sf::Vector2f new_uv = animation_map_["walk_right"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 160, 250));
	}
	else
	{
		const sf::Vector2f new_uv = animation_map_["float_right"]->animate();

		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, 160, 283));
	}
}

/*-------------------------------------------------------------------------
Name	: FloatingCollisions
Purpose	: Checks for when the player collides with water or pits.  Stops 
		  the player from moving through them if it doesn't have the 
		  floating power.  If the player does have the floating power then
		  it will float over these obstacles.  
-------------------------------------------------------------------------*/
void Player::FloatingCollisions(sf::Vector2f temp_position)
{
	// get the position of the top left corner of the player
	sf::Vector2f top_left_corner = getPosition() - (getSize() * 0.5f);

	// if the player isn't currently floating
	if (!floating_)
	{
		// get the map layers
		std::vector<tmx::MapLayer>& layers = map_loader_->GetLayers();

		// loop through all the map layers
		for (auto& layer : layers)
		{
			// if the layer is an object group
			if (layer.type == tmx::ObjectGroup)
			{
				// loop through all the objects on that layer
				for (auto& obj : layer.objects)
				{
					// if the layer is the water collisions layer
					if (layer.name == "water_collisions")
					{
						// loop through all the points
						for (int point_index = 0; point_index < getPointCount(); point_index++)
						{
							// check for collision between the player and the points using the player's top left corner
							if (obj.Contains(top_left_corner + getPoint(point_index)))
							{
								// if the player has the power to float
								if (float_power_)
								{
									// make the player float
									floating_ = true;
								}
								// if the player doesn't have the power yet
								else
								{
									// move the player back so it's not moving over the water
									setPosition(temp_position);
								}
							}
						}
					}
					else if (layer.name == "pit_collisions")
					{
						for (int point_index = 0; point_index < getPointCount(); point_index++)
						{
							if (obj.Contains(top_left_corner + getPoint(point_index)))
							{
								if (pit_float_power_)
								{
									floating_ = true;
								}
								else
								{
									setPosition(temp_position);
								}
							}
						}
					}				
				}
			}
		}
	}
	// if the player is currently floating
	else if (floating_)
	{
		Services::Instance().GetSoundPlayer().Play("player_floating");

		// stop them from floating
		floating_ = false;

		// get the map layers
		std::vector<tmx::MapLayer>& layers = map_loader_->GetLayers();

		// loop through all the map layers
		for (auto& layer : layers)
		{
			// if the layer is an object group
			if (layer.type == tmx::ObjectGroup)
			{
				// loop through all the objects on that layer
				for (auto& obj : layer.objects)
				{
					// if the layer is the water collisions layer
					if (layer.name == "water_collisions")
					{
						// loop through all the points
						for (int point_index = 0; point_index < getPointCount(); point_index++)
						{
							// check for collision between the player and the points using the player's top left corner
							if (obj.Contains(top_left_corner + getPoint(point_index)))
							{
								// make the player float
								floating_ = true;
							}
						}
					}

					if (layer.name == "pit_collisions")
					{
						for (int point_index = 0; point_index < getPointCount(); point_index++)
						{
							if (obj.Contains(top_left_corner + getPoint(point_index)))
							{
								floating_ = true;
							}
						}
					}
				}
			}
		}
	}
}

/*-------------------------------------------------------------------------
END OF FILE
-------------------------------------------------------------------------*/
