/*========================================================================
Class Name	: Section
Authors		: Ewan Duncan, Andrew Hoque
Description	: Contains the logic for an area of the map.
Switches between dead and alive versions and
switches to other sections.  Updates and renders
all game objects.  Uses shaders to make map and
sections transitions smoother.
=======================================================================*/

/*-------------------------------------------------------------------------
INCLUDES
-------------------------------------------------------------------------*/
#include "section.h"

#include <algorithm>

#include "clock.h"
#include "services.h"

std::string Section::destination_door_ = "";
std::string Section::door_direction_ = "";
bool Section::went_through_door_ = false;
bool Section::vignetting_map_ = false;
float Section::vignette_factor_ = 0;
bool Section::fully_vignetted_ = false;

/*-------------------------------------------------------------------------
CONSTRUCTORS
-------------------------------------------------------------------------*/
Section::Section(std::string dead_map_name, std::string living_map_name, tmx::MapLoader* map_loader, Player* player, sf::RenderTexture* render_texture, sf::Sprite* world_sprite, sf::RenderWindow* window):
map_loader_(map_loader),
dead_map_name_(dead_map_name),
living_map_name_(living_map_name),
alive_(false),
visited_(false),
map_switched_(false),
player_(player),
render_texture_(render_texture),
world_sprite_(world_sprite),
window_(window),
pixel_factor_(0),
screen_transition_(false),
fully_pixelated_(false),
new_section_("default")
{
	LoadShaders();
}

/*-------------------------------------------------------------------------
MEMBER FUNCTIONS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Name	: Init
Purpose	: Initialises the section.  
-------------------------------------------------------------------------*/
void Section::Init()
{
	SetUpObjects();
}

/*-------------------------------------------------------------------------
Name	: CompareLayers
Purpose	: Compares two layers.  
-------------------------------------------------------------------------*/
bool CompareLayers( GameObject* lhs,  GameObject* rhs)
{
	return lhs->get_layer() < rhs->get_layer();
}

/*-------------------------------------------------------------------------
Name	: Update
Purpose	: Updates all the game objects in the section.  
-------------------------------------------------------------------------*/
bool Section::Update(std::string &current_section)
{
	// sort the section objects
	std::sort(section_objects_.begin(), section_objects_.end(), CompareLayers);

	// check for an orb pickup
	OrbPickup();

	// loop through the vector of section objects
	for (int current_object = 0; current_object < section_objects_.size();)
	{
		// if an object is active
		if (section_objects_[current_object]->get_active())
		{
			// update it and move onto the next object
			section_objects_[current_object]->Update();
			current_object++;
		}
		// if an object has been deactivated
		else
		{
			// delete it and remove it from the vector
			delete section_objects_[current_object];
			section_objects_.erase(section_objects_.begin() + current_object);
		}
	}

	// check if the section changes
	return ChangeSection(current_section);
}

/*-------------------------------------------------------------------------
Name	: Render
Purpose	: Renders the map and the sections' game objects. 
-------------------------------------------------------------------------*/
void Section::Render()
{
	// render the map to the render texture
	render_texture_->draw(*map_loader_);

	// loop through the vector of section objects
	for (int index = 0; index < section_objects_.size(); index++)
	{
		// if an object is active and drawable
		if (section_objects_[index]->get_active() == true && section_objects_[index]->get_drawable() == true)
		{
			// render it to the render texture
			render_texture_->draw(*section_objects_[index]);
		}
	}
	// render all the map layers to the render texture
	map_loader_->Draw(*render_texture_, tmx::MapLayer::Front, false);
	map_loader_->Draw(*render_texture_, map_loader_->GetLayers().size() - 2 , false);

	// display the render texture
	render_texture_->display();

	// make the game vignette when switching sections
	if (vignetting_map_ == true)
	{
		Vignette();
	}
	// render the game normally when transitions are taking place
	else if (screen_transition_ == false)
	{
		window_->draw(*world_sprite_);
	}
	// make the game pixelate when switching from dead to alive
	else
	{
		Pixelate();
	}
}

/*-------------------------------------------------------------------------
Name	: Pixelate
Purpose	: Pixelates the screen and unpixelates it as the section changes
		  from dead to alive.  
-------------------------------------------------------------------------*/
void Section::Pixelate()
{
	// stop the player from moving during the change
	player_->set_controllable(false);

	// set the pixel threshold on the pixelate shader
	pixelate_shader_.setParameter("pixel_threshold", pixel_factor_);

	// render the game with the pixelate shader
	window_->draw(*world_sprite_, &pixelate_shader_);

	// make the controller vibrate during the change
	Services::Instance().GetInputManager().SetVibration(0, pixel_factor_ * 10, pixel_factor_ * 10);

	// check for when the pixel factor has returned to 0
	if (pixel_factor_ < 0.f)
	{
		// stop the screen transition
		screen_transition_ = false;

		// stop the controller from vibrating
		Services::Instance().GetInputManager().SetVibration(0, 0, 0);

		// return control to the player
		player_->set_controllable(true);
	}

	// when the pixel factor is greater than 0.1
	if (pixel_factor_ > 0.1f)
	{
		// the screen is fully pixelated
		fully_pixelated_ = true;
	}

	// when the screen hasn't yet fully pixelated
	if (fully_pixelated_ == false)
	{
		// increase the pixel factor
		pixel_factor_ += Clock::get_frame_time() * 0.05f;

		// make the music fade out
		Services::Instance().GetMusicPlayer().FadeOut();
	}
	// after the screen has fully pixelated
	else
	{
		// decrease the pixel factor
		pixel_factor_ -= Clock::get_frame_time() * 0.05f;
		
		// make the music fade in
		Services::Instance().GetMusicPlayer().FadeIn();
	}
}

/*-------------------------------------------------------------------------
Name	: Vignette
Purpose	: Vignettes the screen and then returns it to normal as the section 
		  switches to a new section. 
-------------------------------------------------------------------------*/
void Section::Vignette()
{
	// stop the player from moving during the change
	player_->set_controllable(false);

	// make the player walk through the door
	player_->WalkThroughDoor(door_direction_);

	// set the vignette strength on the vignette shader
	vignette_shader_.setParameter("vignette_strength", vignette_factor_);

	// render the game with the vignette shader
	window_->draw(*world_sprite_, &vignette_shader_);

	// when the vignette factor is greater than 1
	if (vignette_factor_ >= 1.f)
	{
		// the screen is fully vignetted
		fully_vignetted_ = true;
	}

	// when the screen hasn't yet fully vignetted
	if (fully_vignetted_ == false)
	{
		// increase the vignette factor
		vignette_factor_ += Clock::get_frame_time() * 0.5f;

		// make the music fade out
		Services::Instance().GetMusicPlayer().FadeOut();
	}
	// after the screen has fully vignetted
	else
	{
		// decrease the vignette factor
		vignette_factor_ -= Clock::get_frame_time() * 0.5f;
		
		// make the music fade in
		Services::Instance().GetMusicPlayer().FadeIn();
	}

	// check for when the vignette factor has been lowered to 0 or less
	if (vignette_factor_ <= 0.f)
	{
		// cap the vignette factor at 0
		vignette_factor_ = 0;

		// stop the vignetting
		vignetting_map_ = false;

		// the screen is no longer vignetting
		fully_vignetted_ = false;

		// return control to the player
		player_->set_controllable(true);
	}
}

/*-------------------------------------------------------------------------
Name	: MovePlayerToDoor
Purpose	: Move the player to the other side of the door in the new section
		  after a switch.  
-------------------------------------------------------------------------*/
void Section::MovePlayerToDoor()
{
	// get the map layers
	std::vector<tmx::MapLayer>& layers = map_loader_->GetLayers();
	
	// loop through all the map layers
	for (auto& layer : layers)
	{
		// if the layer is an object group
		if (layer.type == tmx::ObjectGroup)
		{
			// loop through all the objects on that layer
			for (auto& obj : layer.objects)
			{
				// if the layer doors layer
				if (layer.name == "doors")
				{
					// find the destination door
					if (obj.GetPropertyString("DoorIndex") == destination_door_)
					{
						/*// see which direction the door faces and offset the player in that direction so that 
							they don't immediately collide with the door again //*/

						if(obj.GetPropertyString("Direction") == "up")
						{
							player_->MoveTo(obj.GetCentre().x, obj.GetCentre().y + 100);
						}
						else if (obj.GetPropertyString("Direction") == "down")
						{
							player_->MoveTo(obj.GetCentre().x, obj.GetCentre().y - 100);
						}

						if (obj.GetPropertyString("Direction") == "left")
						{
							player_->MoveTo(obj.GetCentre().x + 100, obj.GetCentre().y);
						}
						else if (obj.GetPropertyString("Direction") == "right")
						{
							player_->MoveTo(obj.GetCentre().x - 100, obj.GetCentre().y);
						}
					}
				}
			}
		}
	}
	
	// the player didn't go through a door
	went_through_door_ = false;
}

/*-------------------------------------------------------------------------
Name	: LoadShaders
Purpose	: Load in the shader .frag files.  
-------------------------------------------------------------------------*/
void Section::LoadShaders()
{
	if (!pixelate_shader_.loadFromFile("pixelate.frag", sf::Shader::Fragment))
	{
		std::cout << "Failed to load pixelate shader" << std::endl;
	}

	pixelate_shader_.setParameter("texture", sf::Shader::CurrentTexture);

	if (!vignette_shader_.loadFromFile("vignette.frag", sf::Shader::Fragment))
	{
		std::cout << "Failed to load vignette shader" << std::endl;
	}

	vignette_shader_.setParameter("texture", sf::Shader::CurrentTexture);

}

/*-------------------------------------------------------------------------
Name	: SetUpObjects
Purpose	: Sets up the objects for the section.  
-------------------------------------------------------------------------*/
void Section::SetUpObjects()
{
	// if this section hasn't been visited
	if (visited_ == false)
	{
		// add the player to the vector of section objects
		section_objects_.push_back(player_);
	}

	// this section has now been visited
	visited_ = true;

	// if the section is in its dead state
	if (alive_ == false)
	{
		// intialise the dead map
		dead_map_.Init(map_loader_, dead_map_name_, &section_objects_);
	}
	// if the section is in its alive state
	else
	{
		// initialise the alive map
		living_map_.Init(map_loader_, living_map_name_, &section_objects_);
	}

	// set the section objects to be the objects currently in use
	Services::Instance().SetObjectsInUse(&section_objects_);
}

/*-------------------------------------------------------------------------
Name	: OrbPickup
Purpose	: Checks for when an orb is picked up.  
-------------------------------------------------------------------------*/
void Section::OrbPickup()
{
	// loop through all the section objects
	for (int current_object = 0; current_object < section_objects_.size(); current_object++)
	{
		// check if the current object is an orb
		if (section_objects_[current_object]->get_type() == kOrb)
		{
			// cast the object pointer as an orb
			Orb* orb = reinterpret_cast<Orb*>(section_objects_[current_object]);
			if (orb->get_collected_status() == true && map_switched_ == false)
			{
				// note that a screen transition needs to be made
				screen_transition_ = true;

				// when the screen has fully pixelated
				if (fully_pixelated_ == true)
				{
					// turn off the dead lights
					DeadToAliveLightsOff();

					// remove objects exclusive to the dead map
					RemoveDeadOnlyObjects();

					// initialise the alive map
					living_map_.Init(map_loader_, living_map_name_, &section_objects_);

					// the section is now in its alive state
					alive_ = true;

					// the map has changed to the alive one
					map_switched_ = true;

					// deactivate the orb
					orb->set_active(false);
				}
			}
		}
	}
}

/*-------------------------------------------------------------------------
Name	: ChangeSection
Purpose	: Switches between sections.  
-------------------------------------------------------------------------*/
bool Section::ChangeSection(std::string &current_section)
{
	// when the section isn't changing
	if (vignetting_map_ == false)
	{
		// get the map layers
		std::vector<tmx::MapLayer>& layers = map_loader_->GetLayers();
		
		// loop through all the map layers
		for (auto& layer : layers)
		{
			// if the layer is an object group
			if (layer.type == tmx::ObjectGroup)
			{
				// loop through all the objects on that layer
				for (auto& obj : layer.objects)
				{
					// if the layer is the doors layer
					if (layer.name == "doors")
					{
						// loop through all the points
						for (int point_index = 0; point_index < player_->getPointCount(); point_index++)
						{
							// check that the player hasn't gone through a door already
							if (went_through_door_ == false)
							{
								// check for collision between the player and the points
								if (obj.Contains(player_->getPosition() + player_->getPoint(point_index)))
								{
									// find out which section the door leads to
									new_section_ = obj.GetPropertyString("SectionName");

									// find out which door it leads to
									destination_door_ = obj.GetPropertyString("DoorIndex");

									// find out the direction of the door
									door_direction_ = obj.GetPropertyString("Direction");

									// note the player went through a door
									went_through_door_ = true;

									// start changing the section by vignetting the map
									vignetting_map_ = true;
									break;
								}
								// if the player didn't hit a door
								else
								{
									// they didn't go through one
									went_through_door_ = false;
								}
							}
						}
					}
				}
			}
		}
	}

	// if the player went through a door and the screen is fully vignetted
	if (went_through_door_ == true && fully_vignetted_ == true)
	{
		// move the player to the corresponding door
		MovePlayerToDoor();
	}

	// if the vignette factor is 1 or more
	if (vignette_factor_ >= 1.f)
	{
		// cap it at 1
		vignette_factor_ = 1.00000f;

		// turn the section lights off
		SectionToSectionLightsOff();

		// change the current section to the new section
		current_section = new_section_;

		// the sections have switched
		return true;
	}
	// otherwise the sections haven't switched yet
	else
	{
		return false;
	}
}

/*-------------------------------------------------------------------------
Name	: DeadToAliveLightsOff
Purpose	: Turns off the lights in the dead map.  
-------------------------------------------------------------------------*/
void Section::DeadToAliveLightsOff()
{
	// loop through all the section objects
	for (int current_object = 0; current_object < section_objects_.size(); current_object++)
	{
		// if the current object has a light
		if (section_objects_[current_object]->get_has_light() == true)
		{
			// turn the light off
			section_objects_[current_object]->TurnLightOff();
		}
	}
}

/*-------------------------------------------------------------------------
Name	: SectionToSectionLightsOff
Purpose	: Turns off the lights in the section.
-------------------------------------------------------------------------*/
void Section::SectionToSectionLightsOff()
{
	// loop through all the section objects
	for (int current_object = 0; current_object < section_objects_.size(); current_object++)
	{
		// if the current object has a light
		if (section_objects_[current_object]->get_has_light() == true)
		{
			// turn the light off
			section_objects_[current_object]->TurnLightOff();
		}
	}
}

/*-------------------------------------------------------------------------
Name	: RemoveDeadOnlyObjects
Purpose	: Removes objects exclusive to the dead map.  
-------------------------------------------------------------------------*/
void Section::RemoveDeadOnlyObjects()
{
	// loop through all the section objects
	for (int current_object = 0; current_object < section_objects_.size(); current_object++)
	{
		// if the object is exclusive to the dead map
		if (section_objects_[current_object]->get_dead_only() == true)
		{
			// deactivate it
			section_objects_[current_object]->set_active(false);
		}
	}
}

/*-------------------------------------------------------------------------
END OF FILE
-------------------------------------------------------------------------*/
