/*========================================================================
Class Name	: Web
Authors		: Ewan Duncan
Description	: Class that represents a web that blocks the player's path
and can be burned by fire balls.
=========================================================================*/

/*-------------------------------------------------------------------------
INCLUDES
-------------------------------------------------------------------------*/
#include "web.h"

#include "animated_prop.h"

/*-------------------------------------------------------------------------
CONSTRUCTORS
-------------------------------------------------------------------------*/
Web::Web(const float width, const float height)
{
	// initialise web properties
	collidable_ = true;
	type_ = kWeb;
	on_fire_ = false;
	drawable_ = true;
	animation_finished_ = false;
	light_created_ = false;
	light_radius_ = width;
	max_light_radius_ = height;
	burned_away_ = false;

	// set up the texture and animation
	AnimatedPropInfo prop_info = *Services::Instance().GetAnimatedPropManager().GetAnimatedPropInfo("burning_web");
	end_uv_.x = prop_info.start_uv.x + ((prop_info.number_of_frames - 1)*  prop_info.dimensions.x);
	end_uv_.y = prop_info.start_uv.y;
	animation_map_["burn"] = new Animation(prop_info.start_uv, end_uv_, prop_info.dimensions, prop_info.animation_speed);
	setTexture(prop_info.texture);
	setSize(prop_info.dimensions);
	setScale(sf::Vector2f(width / prop_info.dimensions.x, height / prop_info.dimensions.y));
	CreateLight(light_radius_, light_radius_, Color3f(0.8f, 0.5f, 0.2f));
	setOrigin(getSize() * 0.5f);
	setTextureRect(sf::IntRect(prop_info.start_uv.x, prop_info.start_uv.y, getSize().x, getSize().y));
}

/*-------------------------------------------------------------------------
MEMBER FUNCTIONS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Name	: Update
Purpose	: Plays the animation when the web is burning and displays a light
		  with a radius that changes in time with the animation.  
-------------------------------------------------------------------------*/
void Web::Update()
{
	// check for when the animation has finished
	if (animation_finished_ == true)
	{
		// deactivate the web
		active_ = false;
	}

	// animate the web if it's on fire
	if (on_fire_ == true)
	{
		// turn the light on if the web has just caught fire
		if (light_created_ == false)
		{
			TurnLightOn();
			light_->SetCenter(Vec2f(getPosition().x, kWindowHeight - getPosition().y));
			light_created_ = true;
		}

		// cap the maxium radius of the light
		if (light_radius_ >= max_light_radius_)
		{
			burned_away_ = true;
		}

		// increase the radius of the light until the web is entirely on fire
		if (burned_away_ == false)
		{
			light_radius_ += (300 * Clock::get_frame_time());
		}
		// decrease the radius of the light as the fire is diminishing
		else
		{
			light_radius_ -= (300 * Clock::get_frame_time());
		}

		// set the radius of the light
		light_->SetRadius(light_radius_);

		// change the uv coordinates to animate the fireball
		const sf::Vector2f new_uv = animation_map_["burn"]->animate();
		setTextureRect(sf::IntRect(new_uv.x, new_uv.y, getSize().x, getSize().y));

		// use the UV coordinates to check when the animation is finished
		if (new_uv.x == end_uv_.x)
		{
			animation_finished_ = true;
		}
	}
}

/*-------------------------------------------------------------------------
END OF FILE
-------------------------------------------------------------------------*/
